"""GitHub API module"""
import logging
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry


class GitHubApi:
    def __init__(self, headers=None, proxy=None):
        self.base_url = 'https://api.github.com'
        self.session = requests.Session()

        retries = Retry(total=3, backoff_factor=1)
        self.session.mount('https://', HTTPAdapter(max_retries=retries))

        if headers:
            self.session.headers.update(headers)
        if proxy:
            self.session.proxies.update(proxy)

        # Rate limits
        self.rate_reset = None
        self.rate_remaining = None

    def __get(self, endpoint, params=None):
        """GET an HTTP resource and return JSON."""
        url = self.base_url + endpoint

        try:
            response = self.session.get(url, params=params)
        except requests.RequestException as e:
            logging.warning(f'Failed to execute GET request: {str(e)}')
            return None

        self.__parse_headers(response.headers)
        return response.json()

    def __parse_headers(self, headers):
        """Parse headers to update rate limits."""
        self.rate_reset = int(headers['x-ratelimit-reset'])
        self.rate_remaining = int(headers['x-ratelimit-remaining'])

    def get_events(self, page=None):
        """Fetch GitHub events."""
        endpoint = '/events'
        params = None

        if page:
            params = {'page': page}

        events = self.__get(endpoint, params)

        return events
